use crate::constant::METADATA_SIZE;

// everything here needs to be adjusted for the deserialized json
//
#[derive(Debug, Clone)]
pub struct Message {
    pub message_kind: MessageKind,
    pub length: u16,
    pub content: String,
}

// idk i just want to layout something, really didn't think much
#[derive(Debug, Clone)]
pub enum MessageKind {
    Move(u8),
    PlayerSetup(u8),
    LobbySetup(u8),
}

impl Message {
    pub fn new(content: impl Into<String>) -> anyhow::Result<Self> {
        let content = content.into();
        let length = content.len() as u16;

        // add message kind to constructor
        Ok(Self {
            length,
            content,
        })
    }

    pub fn encode(&self) -> Vec<u8> {
        let mut buffer = Vec::new();
        buffer.extend(&self.length.to_be_bytes());
        buffer.extend(self.content.as_bytes());

        buffer
    }

    pub fn decode(buffer: &[u8]) -> anyhow::Result<Self> {
        if buffer.len() < METADATA_SIZE {
            return Err(anyhow::anyhow!("Invalid message length"));
        }

        let length = u16::from_be_bytes([buffer[0], buffer[1]]);
        let content = String::from_utf8(buffer[2..2 + length as usize].to_vec())?;

        // add message kind to constructor
        Ok(Self {
            length,
            content,
        })
    }
}
